SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:09'!
test01CartShouldBeEmptyAfterCreation

	| cart priceList |
	
	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	self assert: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:09'!
test02CartShouldNotBeEmptyAfterAddingABook

	| cart book priceList |
	
	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	
	self deny: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:10'!
test03CartShouldNotAllowToAddABookWhichIsNotInTheCatalog

	| cart book priceList |

	book := Book new initializeWith: 123.
	priceList := Dictionary new.	
	cart := Cart new initializeWith: priceList.
	
	self should: [ cart add: book ] raise: Error withExceptionDo: [ :anError | 
		self assert: anError messageText = Cart itemNotInCatalogErrorMessage.
		self assert: cart isEmpty
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:10'!
test04AbookThatIsAddedToTheCartIsInTheCart

	| cart priceList book |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	
	self assert: (cart includes: book)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:23'!
test05CannotCheckoutAnEmptyCart

	| cart priceList cashier creditCard salesBookSize |

	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	creditCard := CreditCard new.
	cashier := Cashier new.
	salesBookSize := cashier salesBook size.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = Cashier emptyCartErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:16'!
test06CheckoutWithOneItemGeneratesASaleWithTheSamePriceAsTheItem

	| cart priceList cashier book creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new.
	creditCard := Object new.
	
	self assert: ((cashier checkout: cart with: creditCard) total = (priceList at: book isbn))! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:13'!
test07CheckoutWithManyItemsGeneratesASaleWithTheSumOfTheItemsPricesAsTotal

	| cart priceList cashier aBook anotherBook creditCard |

	aBook := Book new initializeWith: 123.
	anotherBook := Book new initializeWith: 321.
	priceList := Dictionary newFromPairs: (Array with: aBook isbn with: 5 with: anotherBook isbn with: 8).
	cart := Cart new initializeWith: priceList.
	cart add: aBook.
	cart add: anotherBook.
	cashier := Cashier new.
	creditCard := Object new.
	
	self assert: ((cashier checkout: cart with: creditCard) total = ((priceList at: aBook isbn) + (priceList at: anotherBook isbn)) )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:10'!
test08CheckoutOfAnItemGeneratesASaleThatIncludesTheItem

	| cart priceList cashier book creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new.
	creditCard := Object new.
	
	self assert: ((cashier checkout: cart with: creditCard) includesBook: book isbn)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:12'!
test09ChangesInPriceListDontAffectOlderSalesPrices

	| cart priceList cashier book saleTotal creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new.
	creditCard := Object new.
	saleTotal := (cashier checkout: cart with: creditCard) total.
	priceList at: book isbn put: 8.
	
	self assert: (saleTotal = cashier salesBook last total )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 21:02'!
test10CannotCheckoutWithExpiredCreditCard

	| cart priceList cashier book creditCard salesBookSize |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: MerchantProcessorInterfaceSimulator new.
	salesBookSize := cashier salesBook size.
	creditCard := CreditCard new initializeWith: 123 expirationDate: Date today andOwner: 'Gomez'.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = MerchantProcessorInterface expiredCardMessageText.
		self assert: salesBookSize = cashier salesBook size
	]! !

Object subclass: #Book
	instanceVariableNames: 'isbn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Book methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:47'!
initializeWith: anIsbn

	isbn := anIsbn! !

!Book methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:47'!
isbn

	^isbn! !

Object subclass: #Cart
	instanceVariableNames: 'booksInCart priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:38'!
itemNotInCatalogErrorMessage

	^'El item no se encuentra en el catlogo'! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
add: aBook

	^(priceList includesKey: aBook isbn) ifTrue: [ booksInCart add: aBook ] ifFalse: [ Error signal: self class itemNotInCatalogErrorMessage ]! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:20'!
booksInCart

	^booksInCart! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:51'!
includes: aBook

	^booksInCart includes: aBook! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
initializeWith: aPriceList

	booksInCart := Bag new.
	priceList := aPriceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 17:39'!
isEmpty

	^booksInCart isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
priceList

	^priceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 21:37'!
totalAmountToCheckout

	| total |
	
	total := 0.
	booksInCart do: [ :book | total := total + (priceList at: book isbn) ].
	^total

! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook mpi'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 21:06'!
emptyCartErrorMessage

	^'El carrito est vacio'! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 21:08'!
checkout: aCart with: aCreditCard

	| newSale bookWithPrice |

	mpi validateTransactionWith: aCreditCard andAmount: aCart totalAmountToCheckout.

	aCart isEmpty ifTrue: [ Error signal: self class emptyCartErrorMessage ] ifFalse: [
		newSale := Sale new.
		aCart booksInCart do: [ :book | bookWithPrice := Association key: book isbn value: (aCart priceList at: book isbn). newSale add: bookWithPrice ].
		salesBook add: newSale.
		^newSale
	]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:49'!
initializeWith: aMerchantProcessorInterface

	salesBook := OrderedCollection new.
	mpi := aMerchantProcessorInterface! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 20:24'!
salesBook

	^salesBook! !

Object subclass: #CreditCard
	instanceVariableNames: 'number expirationDate owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CreditCard methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:28'!
initializeWith: aNumber expirationDate: anExpirationDate andOwner: anOwner

	number := aNumber.
	expirationDate := anExpirationDate.
	owner := anOwner! !

Object subclass: #MerchantProcessorAPIClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Object subclass: #MerchantProcessorInterface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

MerchantProcessorInterface subclass: #MerchantProcessorInterfaceSimulator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!MerchantProcessorInterfaceSimulator methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:52'!
validateTransactionWith: aCard andAmount: anAmount

	self validate: aCard! !

Object subclass: #Sale
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:40'!
add: anItem

	items add: anItem.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 18:26'!
includesBook: anIsbn

	^items includesKey: anIsbn! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:38'!
initialize

	items := Dictionary new.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:44'!
total

	| total |
	
	total := 0.
	items do: [ :item | total := total + item value ].
	^total! !
